unit TESH_colors;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons;

type
  TformTESH_colors = class(TForm)
    StyleBox: TListBox;
    IsBoldBox: TCheckBox;
    IsItalicBox: TCheckBox;
    IsUnderlineBox: TCheckBox;
    ForeColorBox: TColorBox;
    PreviewLabel: TLabel;
    BackColorBox: TColorBox;
    ForeLabel: TLabel;
    BackLabel: TLabel;
    procedure StyleBoxClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ForeColorBoxChange(Sender: TObject);
    procedure BackColorBoxChange(Sender: TObject);
    procedure IsBoldBoxClick(Sender: TObject);
    procedure IsItalicBoxClick(Sender: TObject);
    procedure IsUnderlineBoxClick(Sender: TObject);
    procedure ChangeCheckBox(Sender: TObject);
  private
    { Private declarations }
    procedure RefreshLabel;
  public
    { Public declarations }
  end;

const StyleIndex: array[0..11] of byte = (3, 2, 4, 9, 10, 12, 20, 0, 17, 5, 1, 19);

var
  formTESH_colors: TformTESH_colors;

implementation

uses TESH_main;

{$R *.dfm}

procedure TformTESH_colors.StyleBoxClick(Sender: TObject);
begin
  // load the current state of the selected style
  with formTESH_main.CTMain.LanguageManager.LanguageList[1].Styles[StyleIndex[StyleBox.ItemIndex]] do begin
    IsBoldBox.Checked := (fsBold in FontStyles);
    IsItalicBox.Checked := (fsItalic in FontStyles);
    IsUnderlineBox.Checked := (fsUnderline in FontStyles);
    ForeColorBox.Selected := ForeColor;
    BackColorBox.Selected := BackColor;
  end;
  // and refresh the label according to it
  RefreshLabel;
end;

procedure TformTESH_colors.FormActivate(Sender: TObject);
begin
  StyleBox.Selected[0] := true;
  PreviewLabel.Font := formTESH_main.CTMain.Font;
  StyleBox.OnClick(Sender);
end;

procedure TformTESH_colors.ForeColorBoxChange(Sender: TObject);
begin
  Styles[StyleBox.ItemIndex].ForeColor := ForeColorBox.Selected;
  formTESH_main.CTMain.LanguageManager.LanguageList[1].Styles[StyleIndex[StyleBox.ItemIndex]].ForeColor := ForeColorBox.Selected;
  RefreshLabel;
end;

procedure TformTESH_colors.BackColorBoxChange(Sender: TObject);
begin
  Styles[StyleBox.ItemIndex].BackColor := BackColorBox.Selected;
  formTESH_main.CTMain.LanguageManager.LanguageList[1].Styles[StyleIndex[StyleBox.ItemIndex]].BackColor := BackColorBox.Selected;
  RefreshLabel;
end;

procedure TformTESH_colors.RefreshLabel;
begin
  with PreviewLabel do begin
    Font.Style := [];
    if IsBoldBox.Checked then Font.Style := Font.Style+[fsBold];
    if IsItalicBox.Checked then Font.Style := Font.Style+[fsItalic];
    if IsUnderlineBox.Checked then Font.Style := Font.Style+[fsUnderline];
    Font.Color := ForeColorBox.Selected;
    if BackColorBox.Selected = clDefault then Color := clBtnFace
    else Color := BackColorBox.Selected;
  end;
end;

procedure TformTESH_colors.IsBoldBoxClick(Sender: TObject);
begin
  Styles[StyleBox.ItemIndex].IsBold := IsBoldBox.Checked;
  with formTESH_main.CTMain.LanguageManager.LanguageList[1].Styles[StyleIndex[StyleBox.ItemIndex]] do begin
    if IsBoldBox.Checked then FontStyles := FontStyles+[fsBold]
    else FontStyles := FontStyles-[fsBold];
  end;
  RefreshLabel;
end;

procedure TformTESH_colors.IsItalicBoxClick(Sender: TObject);
begin
  Styles[StyleBox.ItemIndex].IsItalic := IsItalicBox.Checked;
  with formTESH_main.CTMain.LanguageManager.LanguageList[1].Styles[StyleIndex[StyleBox.ItemIndex]] do begin
    if IsItalicBox.Checked then FontStyles := FontStyles+[fsItalic]
    else FontStyles := FontStyles-[fsItalic];
  end;
  RefreshLabel;
end;

procedure TformTESH_colors.IsUnderlineBoxClick(Sender: TObject);
begin
  Styles[StyleBox.ItemIndex].IsUnderline := IsUnderlineBox.Checked;
  with formTESH_main.CTMain.LanguageManager.LanguageList[1].Styles[StyleIndex[StyleBox.ItemIndex]] do begin
    if IsUnderlineBox.Checked then FontStyles := FontStyles+[fsUnderline]
    else FontStyles := FontStyles-[fsUnderline];
  end;
  RefreshLabel;
end;

procedure TformTESH_colors.ChangeCheckBox(Sender: TObject);
var
  SenderCB: TCheckBox;
  NewStyle: TFontStyles;
begin
  SenderCB := (Sender as TCheckBox);
  case SenderCB.Tag of
    1: begin
      Styles[StyleBox.ItemIndex].IsBold := SenderCB.Checked;
      NewStyle := [fsBold];
    end;
    2: begin
      Styles[StyleBox.ItemIndex].IsItalic := SenderCB.Checked;
      NewStyle := [fsItalic];
    end;
    3: begin
      Styles[StyleBox.ItemIndex].IsUnderline := SenderCB.Checked;
      NewStyle := [fsUnderline];
    end;
  end;
  with formTESH_main.CTMain.LanguageManager.LanguageList[1].Styles[StyleIndex[StyleBox.ItemIndex]] do begin
    if SenderCB.Checked then FontStyles := FontStyles+NewStyle
    else FontStyles := FontStyles-NewStyle;
  end;
  RefreshLabel;
end;

end.
