unit TESH_options;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, SciLexerMod;

type
  TformTESH_Options = class(TForm)
    UpdateRateEdit: TLabeledEdit;
    UpdateRateUpDown: TUpDown;
    EditorOptions: TGroupBox;
    FontLabel: TLabel;
    FontPreview: TLabel;
    FontButton: TButton;
    LineNumbersCheckBox: TCheckBox;
    EditorFontD: TFontDialog;
    CodeFoldingCheckBox: TCheckBox;
    IdleRateEdit: TLabeledEdit;
    IdleRateUpDown: TUpDown;
    ACCheckBox: TCheckBox;
    FLCheckBox: TCheckBox;
    RefreshRateEdit: TLabeledEdit;
    RefreshRateUpDown: TUpDown;
    AutoRefreshCheck: TCheckBox;
    StyleOButton: TButton;
    LNFontLabel: TLabel;
    LNFontButton: TButton;
    LNFontD: TFontDialog;
    NCUCheckBox: TCheckBox;
    EnableSDCheckBox: TCheckBox;
    procedure ACCheckBoxClick(Sender: TObject);
    procedure FontButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure LineNumbersCheckBoxClick(Sender: TObject);
    procedure CodeFoldingCheckBoxClick(Sender: TObject);
    procedure IdleRateEditChange(Sender: TObject);
    procedure UpdateRateEditChange(Sender: TObject);
    procedure EditorFontDApply(Sender: TObject; Wnd: HWND);
    procedure FLCheckBoxClick(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure AutoRefreshCheckClick(Sender: TObject);
    procedure RefreshRateEditChange(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure StyleOButtonClick(Sender: TObject);
    procedure LNFontButtonClick(Sender: TObject);
    procedure LNFontDApply(Sender: TObject; Wnd: HWND);
    procedure NCUCheckBoxClick(Sender: TObject);
    procedure EnableSDCheckBoxClick(Sender: TObject);
  private
    { Private declarations }
    IsActive: Boolean;
  public
    { Public declarations }
  end;

var
  formTESH_Options: TformTESH_Options;

implementation

uses TESH_main, TESH_colors;

{$R *.dfm}

procedure TformTESH_Options.ACCheckBoxClick(Sender: TObject);
begin
  formTESH_main.AC := ACCheckBox.Checked;
  formTESH_main.SciAutoComplete.Disabled := not ACCheckBox.Checked;
end;

procedure TformTESH_Options.FontButtonClick(Sender: TObject);
begin
  if EditorFontD.Execute then begin
    FontPreview.Font := EditorFontD.Font;
    FontPreview.Caption := EditorFontD.Font.Name;
    formTESH_Main.CTMain.Font := EditorFontD.Font;
  end;
end;

procedure TformTESH_Options.FormActivate(Sender: TObject);
begin
  with formTESH_Main do begin
    FontPreview.Font := CTMain.Font;
    FontPreview.Caption := CTMain.Font.Name;
    EditorFontD.Font := CTMain.Font;
    UpdateRateUpDown.Position := UpdateI;
    IdleRateUpDown.Position := IdleTime;
    RefreshRateUpDown.Position := RefreshI;
    LineNumbersCheckBox.Checked := LineNum;
    CodeFoldingCheckBox.Checked := DoFold;
    ACCheckBox.Checked := AC;
    FLCheckBox.Checked := FL;
    AutoRefreshCheck.Checked := RefreshOn;
    NCUCheckBox.Checked := MegaUndo;
    RefreshRateEdit.Enabled := RefreshOn;
    RefreshRateUpDown.Enabled := RefreshOn;
    with CTMain.LanguageManager.LanguageList[1].Styles[18],LNFontD.Font do begin
      Name := FontName;
      Size := FontSize;
      Color := ForeColor;
      Style := FontStyles;
    end;
  end;
  IsActive := True;
end;

procedure TformTESH_Options.LineNumbersCheckBoxClick(Sender: TObject);
begin
  formTESH_Main.CTMain.Gutter0.Width := Integer(LineNumbersCheckBox.Checked)*30;
  formTESH_Main.LineNum := LineNumbersCheckBox.Checked;
end;

procedure TformTESH_Options.CodeFoldingCheckBoxClick(Sender: TObject);
begin
  if CodeFoldingCheckBox.Checked then formTESH_Main.CTMain.Folding := [foldFold]
  else formTESH_Main.CTMain.Folding := [];
  formTESH_main.DoFold := CodeFoldingCheckBox.Checked;
end;

procedure TformTESH_Options.IdleRateEditChange(Sender: TObject);
var newinterval: cardinal;
begin
  if IsActive then begin
    newinterval := IdleRateUpDown.Position;
    if newinterval < 100 then newinterval := 100;
    formTESH_main.IdleTime := newinterval;
    if formTESH_main.IsIdle then formTESH_main.UpdateTimer.Interval := newinterval;
  end;
end;

procedure TformTESH_Options.UpdateRateEditChange(Sender: TObject);
begin
  if IsActive then begin
    formTESH_main.UpdateI := UpdateRateUpDown.Position;
    if (not formTESH_main.IsIdle) then formTESH_main.UpdateTimer.Interval := formTESH_main.UpdateI;
  end;
end;

procedure TformTESH_Options.EditorFontDApply(Sender: TObject; Wnd: HWND);
begin
  FontPreview.Font := EditorFontD.Font;
  FontPreview.Caption := EditorFontD.Font.Name;
  formTESH_Main.CTMain.Font := EditorFontD.Font;
end;

procedure TformTESH_Options.FLCheckBoxClick(Sender: TObject);
begin
  formTESH_main.FL := FLCheckBox.Checked;
  formTESH_main.SciCallTips.Disabled := not FLCheckBox.Checked;
end;

procedure TformTESH_Options.FormDeactivate(Sender: TObject);
begin
  IsActive := False;
end;

procedure TformTESH_Options.AutoRefreshCheckClick(Sender: TObject);
begin
  formTESH_main.RefreshOn := AutoRefreshCheck.Checked;
  RefreshRateEdit.Enabled := AutoRefreshCheck.Checked;
  RefreshRateUpDown.Enabled := AutoRefreshCheck.Checked;
end;

procedure TformTESH_Options.RefreshRateEditChange(Sender: TObject);
begin
  if IsActive then begin
    formTESH_main.RefreshI := RefreshRateUpDown.Position;
    formTESH_main.RefreshTimer.Interval := formTESH_main.RefreshI;
  end;
end;

procedure TformTESH_Options.FormHide(Sender: TObject);
begin
  formTESH_main.RefreshTimer.Enabled := formTESH_main.RefreshOn;
end;

procedure TformTESH_Options.StyleOButtonClick(Sender: TObject);
begin
  formTESH_colors.Show;
end;

procedure TformTESH_Options.LNFontButtonClick(Sender: TObject);
begin
  if LNFontD.Execute then
    with formTESH_Main.CTMain.LanguageManager.LanguageList[1].Styles[18],LNFontD.Font do begin
      FontName := Name;
      FontSize := Size;
      ForeColor := Color;
      FontStyles := Style;
    end;
end;

procedure TformTESH_Options.LNFontDApply(Sender: TObject; Wnd: HWND);
begin
  with formTESH_Main.CTMain.LanguageManager.LanguageList[1].Styles[18],LNFontD.Font do begin
    FontName := Name;
    FontSize := Size;
    ForeColor := Color;
    FontStyles := Style;
  end;
end;

procedure TformTESH_Options.NCUCheckBoxClick(Sender: TObject);
begin
  formTESH_main.MegaUndo := NCUCheckBox.Checked;
end;

procedure TformTESH_Options.EnableSDCheckBoxClick(Sender: TObject);
begin
  formTESH_Main.EnableSD := EnableSDCheckBox.Checked;
  formTESH_Main.SaveButton.Enabled := EnableSDCheckBox.Checked;
  formTESH_Main.DiscardButton.Enabled := EnableSDCheckBox.Checked;
end;

end.
